/* SRK_bootCreator application to dump the appropriate contents into floppy
 * Author: SRK
 * 
 * To be compiled using gcc as:
 * gcc -o SRK_bootCreator SRK_bootCreator.c
 * 
 * Once compiled, the application can be run as:
 * ./SRK_bootCreator SRK_bootLoader SRK_chainLoader PC-XINU.EXE
 *
 * Running this program requires compiled programs:
 * SRK_bootLoader - nasm compiled code for the MBR. This file has a size restriction of 446 bytes.
 * SRK_chainLoader - nasm compiled code for the SRK chain loader. This file has a size restriction of 1KB.
 * PC-XINU.EXE - Turbo C compiled code for PC-Xinu.
 */

/*
 * SRK bootable floppy will be organized in the following manner:
 * Sector number(s)		FileName
 * 1				SRK_bootLoader + rest of the MBR
 * 2-3				SRK_chainLoader
 * 4-end			PC-XINU.EXE file
 */

//------------------------------------------------------------------------------------------------------
//header files necessary
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include<stdio.h>
#include<stdlib.h>
#include<string.h>

//------------------------------------------------------------------------------------------------------
//definitions
#define FLOPPY_DISC "/dev/fd0"
#define MBR_CODE_SIZE 446
#define CHNLDR_NUM_SEC 2

//------------------------------------------------------------------------------------------------------
// main - program entry point
// inputs - CL arguments as mentioned above

int main(int argc,char **argv)
{
	int floppy_desc,file_desc;
	int i,len,blk_cnt;
	char boot_buf[512];
	struct stat filestat;

	//Check command line structure
	if(argc!=4)
	{
		printf("Usage: %s mbrFile chnLdrFile PC-XinuExe\n",argv[0]);
		return 1;
	}

	//Open Floppy device file
	if((floppy_desc=open(FLOPPY_DISC, O_RDWR))==-1)
	{
		printf("Error opening file: %s\n",FLOPPY_DISC);
		return 1;
	}

	//Open MBR file
	if((file_desc=open(argv[1],O_RDONLY))==-1)
	{
		printf("Error opening file: %s\n",argv[1]);
		return 1;
	}

	//Read File Size
	if(fstat(file_desc,&filestat)==-1)
	{
		printf("Error reading file size\n");
		return 1;
	}

	//Check if file size is valid
	if(filestat.st_size>MBR_CODE_SIZE)
	{
		printf("Error: %s code has to fit into the MBR\n",argv[1]);
		return 1;
	}

	//Read MBR code into the buffer
	lseek(file_desc,0,SEEK_SET);
	printf("Size of MBR code in bytes: %d\n",(len=read(file_desc,boot_buf,512)));

	//Null fill the remaining bytes in the buffer, except the last two
	for(i=len;i<510;i++)
		boot_buf[i]='\0';

	//MBR Signature
	boot_buf[510] = 0x55;
	boot_buf[511] = 0xaa;

	//Write into the floppy
	lseek(floppy_desc, 0, SEEK_SET);
	write(floppy_desc, boot_buf, 512);

	close(file_desc);

	//Open Chain-loader file
	if((file_desc=open(argv[2],O_RDONLY))==-1)
	{
		printf("Error opening file: %s\n",argv[2]);
		return 1;
	}

	//Read file size
	if(fstat(file_desc,&filestat)==-1)
	{
		printf("Error reading file size\n");
		return 1;
	}
	
	//Check if the file size is valid
	blk_cnt=filestat.st_size/512+(filestat.st_size%512?1:0);
	if(blk_cnt!=CHNLDR_NUM_SEC)
	{
		printf("Error: chain loader %s cannot occupy more than 1024 bytes (two sectors)\n",argv[2]);
		return 1;
	}

	//Read file contents into the buffer
	//Write the buffer into the floppy
	lseek(floppy_desc,512,SEEK_SET);
	lseek(file_desc,0,SEEK_SET);
	for(i=0,len=0;i<CHNLDR_NUM_SEC;i++)
	{
		len+=read(file_desc,boot_buf,512);
		write(floppy_desc,boot_buf,512);
	}
	printf("Size of chain loader code in bytes: %d\n",len);
	close(file_desc);

	//Open PC-Xinu EXE file
	if((file_desc=open(argv[3],O_RDONLY))==-1)
	{
		printf("Error opening file: %s\n",argv[3]);
		return 1;
	}

	//Read file size
	if(fstat(file_desc,&filestat)==-1)
	{
		printf("Error reading file size\n");
		return 1;
	}

	//Read file contents into the buffer
	blk_cnt=filestat.st_size/512+(filestat.st_size%512?1:0);
	lseek(file_desc,0,SEEK_SET);
	lseek(floppy_desc,1536,SEEK_SET);
	for(i=0,len=0;i<blk_cnt;i++)
	{
		len+=read(file_desc,boot_buf,512);
		write(floppy_desc,boot_buf,512);
	}
	printf("Size of PC-XINU executable %s in bytes: %d\n",argv[3],len);
	close(file_desc);

	close(floppy_desc);
	return 0;
}

//------------------------------------------------------------------------------------------------------
