;SRK_chainLoader - code that loads and runs the PC-XINU image
;Authors: SRK
;NASM version
;
;-----------------------------------------------------------------------------------------------------------------
;To be compiled using NASM using command
;	nasm -f bin -o SRK_chainLoader.asm
;
;Once compiled, the resulting file goes into the second and third sector of the floppy
;The program argWrite takes care of writing to the floppy in the required format

;-----------------------------------------------------------------------------------------------------------------
;Bootloading PC-Xinu involves the following stages
;1. Setting up the initial stack
;2. Loading the header
;3. Checking whether the file to be loaded is actually an EXE file
;4. Reading the file size
;5. Loading the entire file into the memory
;6. Getting the starting address for the EXE body
;7. Taking care of relocation for the code
;8. Loading the new ISR for INT 0x21
;9. Revectoring interrupt vector INT 0x21
;10. Setting up the Stack taken up by PC-Xinu
;11. Transfering the control of the CPU to the PC-Xinu Code

;-----------------------------------------------------------------------------------------------------------------
;Macro definitions
%define LOC_STK	0x9000
%define LOC_MBR 0x07c0
%define LOC_CHNLDR 0x07e0
%define LOC_HEADER 0x0820

%define FLOPPY_START_SEC 1
%define EXE_START_SEC 4

;-----------------------------------------------------------------------------------------------------------------
;Main program
start:

	push ldsrk		;pass input string
	call putstr		;print string
	pop ax			;get rid of pushed arg

;1
	mov ax,LOC_STK
	mov ss,ax
	mov sp,0		;initial stack setup at LOC_STK:0

	push stksetup		;pass input string
	call putstr		;print string
	pop ax			;get rid of pushed arg

;2
	push ldheader		;pass input string
	call putstr		;print string
	pop ax			;get rid of pushed arg

	mov ax,LOC_HEADER
	mov es,ax
	mov bx,0		;header load location at LOC_HEADER:0

	mov dl,0		;drive number
	mov dh,0		;head number
	mov ch,0		;cylinder number
	mov cl,EXE_START_SEC	;starting sector
	mov al,1		;number of sectors to read
	mov ah,2
	int 0x13		;read disk

;3
	push execheck		;pass input string
	call putstr		;print string
	pop ax			;get rid of pushed arg

	mov ax,LOC_HEADER
	mov ds,ax		;ds points to header location
	cmp word [0],0x5a4d	;check for EXE 'MZ' signature
	je .carryon

	push sigerr		;pass input string
	call putstr		;print string
	pop ax			;get rid of pushed arg
	jmp $			;infinite loop

.carryon:
;4,5
	push rdfilesize		;pass input string
	call putstr		;print string
	pop ax			;get rid of pushed arg

	mov ax,LOC_MBR
	mov es,ax
	mov bx,0		;floppy load location at LOC_MBR:0

	mov ax,LOC_HEADER
	mov ds,ax		;ds points to header location
	mov ax,[0x04]		;number of sectors to be loaded
	add ax,3		;MBR and chain-loader also to be loaded
	mov dl,0		;drive number
	mov dh,0		;head number
	mov ch,0		;cylinder number
	mov cl,FLOPPY_START_SEC	;starting sector number
.secload:
	push ax			;NOTE: push does not change flags register
	mov al,1		;number of sectors
	mov ah,2		;INT 13H disk read subfunction
	int 0x13
	jnc .go

	push dskrderr		;pass input string
	call putstr		;print string
	pop ax			;get rid of pushed arg
	jmp $

.go:
	inc cl			;increment sector number
	cmp cl,18		;check if sector number is valid
	jle .samecyl		;if valid jump ahead

	mov cl,1		;else, reset start sector
	cmp dh,0		;check for surface number
	jz .fliponly		;if surface zero, do not increment cylinder number
	inc ch			;else, increment cylinder number
.fliponly:
	xor dh,1		;flip head number

.samecyl:
	mov ax,es
	add ax,0x20
	mov es,ax		;increment es:bx by 512 bytes
	mov bx,0

	pop ax
	dec ax			;decrement sectors to load
	jnz .secload

	push ldfile		;pass input string
	call putstr		;print string
	pop ax			;get rid of pushed arg

;6
	mov ax,LOC_HEADER
	mov ds,ax		;ds points to header location

	mov dx,[0x08]
	add dx,ax		;dx now contains the segment pointer to start of EXE body

;7
	push relocating		;pass input string
	call putstr		;print string
	pop ax			;get rid of pushed arg

	mov cx,[0x06]
	mov bx,[0x18]		;offset of the first relocation address

	mov ax,LOC_HEADER
.reloc:
	mov ds,ax		;reset ds to point to header
	add bx,2		;move to segment part in the entry
	add [bx],dx		;change relocation table entry to point to segment relative to load location
	sub bx,2		;move back to offset part
	lds si,[bx]		;ds:si points to the place where relocation needs to be done
	add [si],dx		;do relocation
	add bx,4		;increment to next entry
	loop .reloc

;8
	push loadisr		;pass input string
	call putstr		;print string
	pop ax			;get rid of pushed arg

	mov ax,LOC_HEADER
	mov ds,ax		;ds points to header location

	mov bx,0xcf		;machine instruction IRET
	mov [0],bx
	
;9
	push revectoring	;pass input string
	call putstr		;print string
	pop ax			;get rid of pushed arg

	mov ax,0
	mov ds,ax		;ds points to interrupt vector table

	mov bx,0x21		;INT 21H to be revectored
	shl bx,2		;INT 21H entry is at 0x21 * 4
	mov word [bx],0		;offset 0
	add bx,2
	mov word [bx],LOC_HEADER	;segment LOC_HEADER

;10
	push ldregis		;pass input string
	call putstr		;print string
	pop ax			;get rid of pushed arg

	mov ax,LOC_HEADER
	mov ds,ax		;ds points to header location

	mov bx,[0x0e]		;ss value
	add bx,dx		;relocated value
	mov ss,bx

	mov sp,[0x10]		;sp value

;11
	mov ax,LOC_HEADER
	mov ds,ax		;ds points to header location

	mov bx,[0x16]		;cs value
	add bx,dx		;relocated value

	pushf			;flag values when EXE takes over
	push bx			;EXE segment location
	push word [0x14]	;EXE offset location

	;initializations before the control is tranferred
	mov ax,[0x04]
	mov bx,0x200
	mul bx
	cmp word [0x02],0
	je  .reset

	mov bx,0x200
	sub bx,[0x02]
	sub ax,bx
	jnc .reset

	dec dx			;dx/ax contains HIGH/LOW word values for the number of bytes in the EXE

.reset:
	mov bx,dx
	mov cx,ax		;move dx/ax into bx/cx

	mov ax,LOC_HEADER
	mov ds,ax		;ds contains EXE file start segment
	mov es,ax		;es contains EXE file start segment

	xor ax,ax
	xor dx,dx		;clear ax and dx registers

	iret			;this is a method to transfer control (push flags,cs,ip values. iret pops them)

;-----------------------------------------------------------------------------------------------------------------
;Function to print a string to screen
;Takes as input the address of input string
putstr:
	push bp
	mov bp,sp
	push ax
	push bx
	push si
	push ds

	mov ax,LOC_CHNLDR
	mov ds,ax

	mov si,[bp+4]		;get the pushed argument
.start:
	lodsb			;load ds:si to al
	or al,al
	jz .done		;do operation till al does not point to '\0'

	mov bx,0x0007		;character attributes
	mov ah,0x0e		;terminal output subfunction
	int 0x10		;print character
	jmp .start
.done:
	pop ds
	pop si
	pop bx
	pop ax
	pop bp
	retn

;-----------------------------------------------------------------------------------------------------------------
;All the Messages to be printed
ldsrk		db 13,10,'SRK LOADING STARTED... ',0
stksetup	db 13,10,'Setting up initial Stack ',0
ldheader	db 13,10,'Loading header ',0
execheck	db 13,10,'Checking EXE signature ',0
sigerr		db 13,10,'File corrupt or not an exe ',0
rdfilesize	db 13,10,'Reading the size of PC-Xinu Code ',0
dskrderr	db 13,10,'Error reading floppy disk ',0
ldfile		db 13,10,'EXE file read successful ',0
relocating	db 13,10,'Relocating PC-Xinu code ',0
loadisr		db 13,10,'Loading new ISR ',0
revectoring	db 13,10,'Revectoring to new ISR ',0
ldregis		db 13,10,'Loading appropriate values into registers ',0
;-----------------------------------------------------------------------------------------------------------------
